/* OLPC machine specific definitions */

#ifndef _ASM_X86_OLPC_H
#define _ASM_X86_OLPC_H

#include <asm/geode.h>

struct olpc_platform_t {
	int flags;
	uint32_t boardrev;
	int ecver;
};

#define OLPC_F_PRESENT		0x01
#define OLPC_F_DCON		0x02
#define OLPC_F_VSA		0x04

/*
 * for XO-1, the revision in hex is equivalent to the common name
 * for the revision -- i.e. board B3 has revision 0xb3.  with the
 * introduction of XO 1.5 boards, the EC will no longer increment
 * the first nibble of the ID, since otherwise we'll run out of
 * namespace too soon.  so we'll start mapping with macros from
 * 0xd0 onwards.
 */
#define BOARD_XO_1_B1	0xb1
#define BOARD_XO_1_B2	0xb2
#define BOARD_XO_1_B3	0xb3
#define BOARD_XO_1_C1	0xc1
#define BOARD_XO_1_C2	0xc2

#define BOARD_XO_1_5_Ax	0xd0   // A0 and A1 boards have the same ID
#define BOARD_XO_1_5_B1	0xd1
#define BOARD_XO_1_5_B2	0xd1   // B2 are also 0xd1.  all B1 were destroyed.
#define BOARD_XO_1_5_B3	0xd2
#define BOARD_XO_1_5_C1	0xd3   // initial factory build (12/2009)
#define BOARD_XO_1_5_C2	0xd4   // ramp production (2/2010)

#ifdef CONFIG_OLPC

extern struct olpc_platform_t olpc_platform_info;

/*
 * OLPC board IDs contain the major build number within the mask 0x0ff0,
 * and the minor build number withing 0x000f.  Pre-builds have a minor
 * number less than 8, and normal builds start at 8.  For example, 0x0B10
 * is a PreB1, and 0x0C18 is a C1.
 */

static inline uint32_t olpc_board(uint8_t id)
{
	return (id << 4) | 0x8;
}

static inline uint32_t olpc_board_pre(uint8_t id)
{
	return id << 4;
}

static inline int machine_is_olpc(void)
{
	return (olpc_platform_info.flags & OLPC_F_PRESENT) ? 1 : 0;
}

/*
 * The DCON is OLPC's Display Controller.  It has a number of unique
 * features that we might want to take advantage of..
 */
static inline int olpc_has_dcon(void)
{
	return (olpc_platform_info.flags & OLPC_F_DCON) ? 1 : 0;
}

/*
 * The VSA is software from AMD that typical Geode bioses will include.
 * It is used to emulate the PCI bus, VGA, etc.  OLPC's Open Firmware does
 * not include the VSA; instead, PCI is emulated by the kernel.
 *
 * The VSA is described further in arch/x86/pci/olpc.c.
 */
static inline int olpc_has_vsa(void)
{
	return (olpc_platform_info.flags & OLPC_F_VSA) ? 1 : 0;
}

/*
 * The "Mass Production" version of OLPC's XO is identified as being model
 * C2.  During the prototype phase, the following models (in chronological
 * order) were created: A1, B1, B2, B3, B4, C1.  The A1 through B2 models
 * were based on Geode GX CPUs, and models after that were based upon
 * Geode LX CPUs.  There were also some hand-assembled models floating
 * around, referred to as PreB1, PreB2, etc.
 */
static inline int olpc_board_at_least(uint32_t rev)
{
	return olpc_platform_info.boardrev >= rev;
}

#else

static inline int machine_is_olpc(void)
{
	return 0;
}

static inline int olpc_has_dcon(void)
{
	return 0;
}

static inline int olpc_has_vsa(void)
{
	return 0;
}

#endif

/* EC related functions */

extern int olpc_ec_cmd(unsigned char cmd, unsigned char *inbuf, size_t inlen,
		unsigned char *outbuf, size_t outlen);

/* EC commands */
#define EC_FIRMWARE_REV		0x08	/* uint8_t */
#define EC_BAT_VOLTAGE		0x10	/* uint16_t, *9.76/32, mV   */
#define EC_BAT_CURRENT		0x11	/* int16_t, *15.625/120, mA   */
#define EC_BAT_ACR		0x12	/* int16_t, *6250/15, Ah  */
#define EC_BAT_TEMP		0x13	/* uint16_t, *100/256, C  */
#define EC_AMB_TEMP		0x14	/* uint16_t, *100/256, C  */
#define EC_BAT_STATUS		0x15	/* uint8_t, bitmask */
#define EC_BAT_SOC		0x16	/* uint8_t, percentage */
#define EC_BAT_SERIAL		0x17	/* uint8_t[6] */
#define EC_BAT_EEPROM		0x18	/* uint8_t addr input, uint8_t output */
#define EC_WRITE_SCI_MASK	0x1b	/* uint8_t, bitmask */
#define EC_READ_SCI_MASK	0x1c	/* uint8_t, bitmask */
#define EC_BAT_ERRCODE		0x1f	/* uint8_t, bitmask */
#define EC_WAKE_UP_WLAN		0x24	/* null */
#define EC_WLAN_LEAVE_RESET	0x25	/* null */
#define EC_READ_EB_MODE		0x2a	/* uint8_t */
#define EC_SET_SCI_INHIBIT	0x32	/* null */
#define EC_SET_SCI_INHIBIT_RELEASE	0x34	/* null */
#define EC_WLAN_ENTER_RESET	0x35	/* null */
#define EC_SET_GP_WAKEUP	0x36    /* 4 * uint8_t, big end first */
#define EC_READ_EXT_SCI_MASK	0x37	/* uint16_t, bitmask */
#define EC_WRITE_EXT_SCI_MASK	0x38	/* uint16_t, bitmask */
#define EC_CLEAR_GP_WAKEUP	0x39    /* null */
#define EC_SCI_QUERY		0x84	/* uint8_t */
#define EC_EXT_SCI_QUERY	0x85	/* uint16_t */

/* results from EC_BAT_STATUS */
#define BAT_STAT_PRESENT	0x01
#define BAT_STAT_FULL		0x02
#define BAT_STAT_LOW		0x04
#define BAT_STAT_DESTROY	0x08
#define BAT_STAT_AC		0x10
#define BAT_STAT_CHARGING	0x20
#define BAT_STAT_DISCHARGING	0x40
#define BAT_STAT_TRICKLE	0x80


/* results from EC_BAT_ERRCODE */
#define BAT_ERR_INFOFAIL	0x02
#define BAT_ERR_OVERVOLTAGE	0x04
#define BAT_ERR_OVERTEMP	0x05
#define BAT_ERR_GAUGESTOP	0x06
#define BAT_ERR_OUT_OF_CONTROL	0x07
#define BAT_ERR_ID_FAIL		0x09
#define BAT_ERR_ACR_FAIL	0x10


/* SCI source values -- results from EC_SCI_QUERY */
#define EC_SCI_SRC_EMPTY	0x00
#define EC_SCI_SRC_GAME		0x01
#define EC_SCI_SRC_BATTERY	0x02
#define EC_SCI_SRC_BATSOC	0x04
#define EC_SCI_SRC_BATERR	0x08
#define EC_SCI_SRC_EBOOK	0x10   // XO-1 only
#define EC_SCI_SRC_WLAN		0x20   // XO-1 only
#define EC_SCI_SRC_ACPWR	0x40
#define EC_SCI_SRC_BATCRIT	0x80
#define EC_SCI_SRC_GPWAKE      0x100   // XO-1.5 only
#define EC_SCI_SRC_ALL         0x1FF

/* GPIO assignments */


#if defined(CONFIG_OLPC_XO_1)

#define OLPC_GPIO_MIC_AC	geode_gpio(1)
#define OLPC_GPIO_DCON_IRQ	geode_gpio(7)
#define OLPC_GPIO_THRM_ALRM	geode_gpio(10)
#define OLPC_GPIO_SMB_CLK	geode_gpio(14)
#define OLPC_GPIO_SMB_DATA	geode_gpio(15)
#define OLPC_GPIO_WORKAUX	geode_gpio(24)
#define OLPC_GPIO_LID		geode_gpio(26)
#define OLPC_GPIO_ECSCI		geode_gpio(27)

#elif defined(CONFIG_OLPC_XO_1_5)

/* Mapping between numeric GPIO ID and the actual GPIO hardware numbering:
 * (see vx855-gpio.c)
 * 0..13	GPI 0..13
 * 14..26	GPO 0..12
 * 27..41	GPIO 0..14
 */
#define NR_VX855_GPI	14
#define NR_VX855_GPO	13
#define NR_VX855_GPIO	15

#define VX855_GPI(n)	(n)
#define VX855_GPO(n)	(NR_VX855_GPI + (n))
#define VX855_GPIO(n)	(NR_VX855_GPI + NR_VX855_GPO + (n))

#endif

#if defined(CONFIG_OLPC_XO_1_PM) || defined(CONFIG_OLPC_XO_1_5_PM)
void olpc_register_battery_callback(void (*f)(unsigned long));
void olpc_deregister_battery_callback(void);
#else
#define olpc_register_battery_callback(f)
#define olpc_deregister_battery_callback()
#endif

#ifdef CONFIG_OLPC_XO_1_PM
void olpc_ec_disable_wlan(void);
void olpc_ec_enable_wlan(void);
int olpc_ec_wlan_enabled(void);
#else
#define olpc_ec_disable_wlan()
#define olpc_ec_enable_wlan()
#define olpc_ec_wlan_enabled()
#endif

#endif /* _ASM_X86_OLPC_H */
